@@ -11,6 +11,7 @@
 /*                                                                       */
 /*************************************************************************/
 
+#include "anysplit.h"
 #include "dict-api.h"
 #include "dict-common.h"
 #include "externs.h"
@@ -25,10 +26,6 @@
 #include "dict-file/read-dict.h"
 #include "dict-file/word-file.h"
 
-#ifdef USE_ANYSPLIT
-#include "anysplit.h"
-#endif
-
 /* ======================================================================== */
 /* Affix type finding */
 
@@ -37,7 +34,7 @@
  *
  * Suffixes have the form =asdf.asdf or possibly just =asdf without
  * the dot (subscript mark). The "null" suffixes have the form
- * =.asdf (always with the ubscript mark, as there are several).
+ * =.asdf (always with the subscript mark, as there are several).
  * Ordinary equals signs appearing in regular text are either = or =[!].
  */
 bool is_suffix(const char infix_mark, const char* w)
@@ -275,7 +272,7 @@ int delete_dictionary_words(Dictionary dict, const char * s)
 /**
  * The following two functions free the Exp s and the
  * E_lists of the dictionary.  Not to be confused with
- * free_E_list in utilities.c
+ * free_E_list in word-utils.c.
  */
 static void free_Elist(E_list * l)
 {
@@ -336,6 +333,7 @@ static void affix_list_delete(Dictionary dict)
 		if (atc->string) free(atc->string);
 	}
 	free(dict->afdict_class);
+	dict->afdict_class = NULL;
 }
 
 void dictionary_delete(Dictionary dict)
@@ -364,9 +362,7 @@ void dictionary_delete(Dictionary dict)
 	pp_knowledge_close(dict->hpsg_knowledge);
 	string_set_delete(dict->string_set);
 	free_regexs(dict->regex_root);
-#ifdef USE_ANYSPLIT
 	free_anysplit(dict);
-#endif
 	free_dictionary(dict);
 	xfree(dict, sizeof(struct Dictionary_s));
 	object_open(NULL, NULL, NULL); /* Free the directory path cache */
@@ -418,6 +414,7 @@ void print_expression(Exp * n)
 
 #else /* INFIX_NOTATION */
 
+#define COST_FMT "%.3f"
 /**
  * print the expression, in infix-style
  */
@@ -425,6 +422,7 @@ static void print_expression_parens(const Exp * n, int need_parens)
 {
 	E_list * el;
 	int i, icost;
+	double dcost;
 
 	if (n == NULL)
 	{
@@ -433,6 +431,17 @@ static void print_expression_parens(const Exp * n, int need_parens)
 	}
 
 	icost = (int) (n->cost);
+	dcost = n->cost - icost;
+	if (dcost > 10E-4)
+	{
+		dcost = n->cost;
+		icost = 1;
+	}
+	else
+	{
+		dcost = 0;
+	}
+
 	/* print the connector only */
 	if (n->type == CONNECTOR_type)
 	{
@@ -440,6 +449,7 @@ static void print_expression_parens(const Exp * n, int need_parens)
 		if (n->multi) printf("@");
 		printf("%s%c", n->u.string, n->dir);
 		for (i=0; i<icost; i++) printf("]");
+		if (0 != dcost) printf(COST_FMT, dcost);
 		return;
 	}
 
@@ -450,6 +460,7 @@ static void print_expression_parens(const Exp * n, int need_parens)
 		for (i=0; i<icost; i++) printf("[");
 		printf ("()");
 		for (i=0; i<icost; i++) printf("]");
+		if (0 != dcost) printf(COST_FMT, dcost);
 		return;
 	}
 
@@ -473,6 +484,7 @@ static void print_expression_parens(const Exp * n, int need_parens)
 	if ((n->type == AND_type) && (el->next == NULL))
 	{
 		for (i=0; i<icost; i++) printf("]");
+		if (0 != dcost) printf(COST_FMT, dcost);
 		if ((icost == 0) && need_parens) printf(")");
 		return;
 	}
@@ -501,6 +513,7 @@ static void print_expression_parens(const Exp * n, int need_parens)
 	}
 
 	for (i=0; i<icost; i++) printf("]");
+	if (0 != dcost) printf(COST_FMT, dcost);
 	if ((icost == 0) && need_parens) printf(")");
 }
 
