@@ -1,6 +1,7 @@
-
+void remap_linkages(Linkage, const int *remap);
 void compute_chosen_words(Sentence, Linkage, Parse_Options);
-void linkage_free_pp_info(Linkage);
 
 void partial_init_linkage(Linkage, unsigned int N_words);
 void check_link_size(Linkage);
+void remove_empty_words(Linkage);
+void free_linkage(Linkage);
