@@ -19,6 +19,7 @@
 #include "print.h"
 #include "prune.h"
 #include "resources.h"
+#include "string-set.h"
 #include "structures.h"
 #include "word-utils.h"
 
@@ -26,12 +27,17 @@ static void
 set_connector_list_length_limit(Connector *c,
                                 Connector_set *conset,
                                 int short_len,
-                                Parse_Options opts)
+                                bool all_short,
+                                const char * ZZZ)
 {
 	for (; c!=NULL; c=c->next)
 	{
-		if (opts->all_short ||
-		    (conset != NULL && !match_in_connector_set(conset, c)))
+		if (string_set_cmp (ZZZ, c->string))
+		{
+			c->length_limit = 1;
+		}
+		else if (all_short ||
+		         (conset != NULL && !match_in_connector_set(conset, c)))
 		{
 			c->length_limit = short_len;
 		}
@@ -43,16 +49,28 @@ set_connector_length_limits(Sentence sent, Parse_Options opts)
 {
 	size_t i;
 	unsigned int len = opts->short_length;
+	bool all_short = opts->all_short;
 	Connector_set * ucs = sent->dict->unlimited_connector_set;
+	const char * ZZZ = string_set_add("ZZZ", sent->dict->string_set);
+
+	if (0)
+	{
+		/* Not setting the length_limit saves observable time. However, if we
+		 * would like to set the ZZZ connector length_limit to 1 for all
+		 * sentences, we cannot do the following.
+		 * FIXME(?): Use a flag that the sentence contains an empty word. */
+		if (len >= sent->length) return; /* No point to enforce short_length. */
+	}
 
-	if (len >= sent->length) return; /* No point to enforce short_length. */
 	if (len > UNLIMITED_LEN) len = UNLIMITED_LEN;
 
-	for (i=0; i<sent->length; i++) {
+	for (i=0; i<sent->length; i++)
+	{
 		Disjunct *d;
-		for (d = sent->word[i].d; d != NULL; d = d->next) {
-			set_connector_list_length_limit(d->left, ucs, len, opts);
-			set_connector_list_length_limit(d->right, ucs, len, opts);
+		for (d = sent->word[i].d; d != NULL; d = d->next)
+		{
+			set_connector_list_length_limit(d->left, ucs, len, all_short, ZZZ);
+			set_connector_list_length_limit(d->right, ucs, len, all_short, ZZZ);
 		}
 	}
 }
