@@ -22,3 +22,5 @@ void   print_sentence_word_alternatives(Sentence sent, bool debugprint,
        void (*display)(Dictionary, const char *), struct tokenpos *);
 void print_with_subscript_dot(const char *);
 void print_chosen_disjuncts_words(const Linkage);
+void print_lwg_path(Gword **);
+void print_wordgraph_pathpos(const Wordgraph_pathpos *);
