@@ -18,6 +18,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdarg.h>
 
 #ifndef _WIN32
 #include <unistd.h>
@@ -542,7 +543,7 @@ dyn_str* dyn_str_new(void)
 	ds->str[0] = 0x0;
 	return ds;
 }
- 
+
 void dyn_str_delete(dyn_str* ds)
 {
 	free(ds->str);
@@ -560,7 +561,6 @@ void dyn_strcat(dyn_str* ds, const char *str)
 	strcpy (ds->str+ds->end, str);
 	ds->end += l;
 }
- 
 
 /* =========================================================== */
 /* File path and dictionary open routines below */
@@ -704,11 +704,12 @@ char * dictionary_get_data_dir(void)
 		data_dir = safe_strdup(custom_data_dir);
 		return data_dir;
 	}
-	
+
 #ifdef ENABLE_BINRELOC
 	data_dir = safe_strdup (BR_DATADIR("/link-grammar"));
 #elif defined(_WIN32)
-	/* Dynamically locate library and return containing directory */
+	/* Dynamically locate library and return containing directory.
+	 * FIXME: A rewrite is needed. */
 	hInstance = GetModuleHandle("link-grammar.dll");
 	if (hInstance != NULL)
 	{
@@ -734,27 +735,33 @@ char * dictionary_get_data_dir(void)
 			if (GetModuleFileName(hInstance, prog_path16, MAX_PATH))
 			{
 				char * data_dir16 = NULL;
-				// convert 2-byte to 1-byte encoding of prog_path
-				char prog_path[MAX_PATH/2];
-				int i, k;
-				for (i = 0; i < MAX_PATH; i++)
-				{
-					k = i + i;
-					if (prog_path16[k] == 0 )
-					{
-						prog_path[i] = 0;
-						break; // found the string ending null char
-					}
-					// XXX this cannot possibly be correct for UTF-16 filepaths!! ?? FIXME
-					prog_path[i] = prog_path16[k];
-				}
-#ifdef _DEBUG
-				prt_error("Info: GetModuleFileName=%s", (prog_path ? prog_path : "NULL"));
-#endif
+
 				if (sizeof(TCHAR) == 1)
+				{
 				    data_dir16 = path_get_dirname(prog_path16);
+				}
 				else
+				{
+					// convert 2-byte to 1-byte encoding of prog_path
+					char prog_path[MAX_PATH/2];
+
+					int i, k;
+					for (i = 0; i < MAX_PATH; i++)
+					{
+						k = i + i;
+						if (prog_path16[k] == 0 )
+						{
+							prog_path[i] = 0;
+							break; // found the string ending null char
+						}
+						// XXX this cannot possibly be correct for UTF-16 filepaths!! ?? FIXME
+						prog_path[i] = prog_path16[k];
+					}
+#ifdef _DEBUG
+					prt_error("Info: GetModuleFileName=%s", (prog_path ? prog_path : "NULL"));
+#endif
 				    data_dir16 = path_get_dirname(prog_path);
+				}
 				if (data_dir16 != NULL)
 				{
 					printf("   found dir for current prog %s\n", data_dir16);
@@ -784,7 +791,7 @@ char * dictionary_get_data_dir(void)
  * Otherwise, it looks for the file in a sequence of directories, as
  * specified in the dictpath array, until it finds it.
  *
- * If it s still not found, it may be that the user specified a relative
+ * If it is still not found, it may be that the user specified a relative
  * path, so it tries to open the exact file.
  *
  * Associated data files are looked in the *same* directory in which the
@@ -824,7 +831,7 @@ void * object_open(const char *filename,
 	/* Look for absolute filename.
 	 * Unix: starts with leading slash.
 	 * Windows: starts with C:\  except that the drive letter may differ.
-	 * Note that only native windows C library uses backslashs; mingw 
+	 * Note that only native windows C library uses backslashes; mingw
 	 * seems to use forward-slash, from what I can tell.
 	 */
 	if ((filename[0] == '/')
@@ -934,7 +941,7 @@ bool file_exists(const char * dict_name)
 	fstat(fd, &buf);
 	if (0 < buf.st_size) retval = true;
 
-  	fclose(fp);
+	fclose(fp);
 	return retval;
 }
 
@@ -980,7 +987,7 @@ char *get_file_contents(const char * dict_name)
 			 break;
 	}
 
-  	fclose(fp);
+	fclose(fp);
 
 	if (left < 0)
 	{
@@ -1122,12 +1129,56 @@ char * get_default_locale(void)
 	return locale;
 }
 
+/* ============================================================= */
+/* MSVC: Utilities for converting %zu to %Iu */
+
+#ifdef _MSC_VER
+int printf_compat(const char *format, ...)
+{
+	char * tmp = alloca(strlen(format)+1);
+	char * tok = tmp;
+	va_list args;
+
+	strcpy(tmp, format);
+	while ((tok = strstr(tok, "%zu"))) { tok[1] = 'I'; tok++;}
+	va_start(args, format);
+	return vprintf(tmp, args); /* fine on MSVC to return before va_end() */
+	va_end(args);
+}
+
+int fprintf_compat(FILE *file, const char *format, ...)
+{
+	char * tmp = alloca(strlen(format)+1);
+	char * tok = tmp;
+	va_list args;
+
+	strcpy(tmp, format);
+	while ((tok = strstr(tok, "%zu"))) { tok[1] = 'I'; tok++;}
+	va_start(args, format);
+	return vfprintf(file, tmp, args); /* fine on MSVC to return before va_end() */
+	va_end(args);
+}
+
+int sprintf_compat(char *str, const char *format, ...)
+{
+	char * tmp = alloca(strlen(format)+1);
+	char * tok = tmp;
+	va_list args;
+
+	strcpy(tmp, format);
+	while ((tok = strstr(tok, "%zu"))) { tok[1] = 'I'; tok++;}
+	va_start(args, format);
+	return vsprintf(str, tmp, args); /* fine on MSVC to return before va_end() */
+	va_end(args);
+}
+#endif /* _MSC_VER */
+
 /* ============================================================= */
 /* Alternatives utilities */
 
 static const char ** resize_alts(const char **arr, size_t len)
 {
-	arr = (const char **)realloc(arr, (len+2) * sizeof(const char *));
+	arr = realloc(arr, (len+2) * sizeof(char *));
 	arr[len+1] = NULL;
 	return arr;
 }
@@ -1142,10 +1193,10 @@ size_t altlen(const char **arr)
 
 void altappend(Sentence sent, const char ***altp, const char *w)
 {
-	int n = altlen(*altp);
+	size_t n = altlen(*altp);
 
 	*altp = resize_alts(*altp, n);
 	(*altp)[n] = string_set_add(w, sent->string_set);
 }
 
-/* ========================== END OF FILE =================== */
+/* ============================================================= */
